<!DOCTYPE html>
<html>
<head>
  <title>Package Index</title>
</head>
<body>
  {% for filename, url in pkgs.iteritems() %}
    <a href="{{ url }}">{{ filename }}</a><br>
  {%- endfor %}
</body>
</html>
